@@ -19,7 +19,16 @@ using namespace boost::spirit;
 
 namespace conf4cpp
 {
-    enum ti_atomic_t { TI_BOOL, TI_INT, TI_DOUBLE, TI_STRING };
+    enum ti_atomic_t {
+        TI_BOOL,
+        TI_INT,
+        TI_UINT,
+        TI_DOUBLE,
+        TI_STRING,
+        TI_TIME,
+        TI_IPV4ADDR,
+        TI_IPV6ADDR,
+    };
     struct ti_enum_t {
 	ti_enum_t(const string& eid_) : eid(eid_) {}
 	string eid;
@@ -46,10 +55,15 @@ namespace conf4cpp
 	type_checker(const var_t& v) : v_(v) {}
 	error_t operator() (ti_atomic_t ta) const {
 	    switch (ta) {
-	    case TI_BOOL:   if (is_bool(v_))   return error_none; break;
-	    case TI_INT:    if (is_int(v_))    return error_none; break;
-	    case TI_DOUBLE: if (is_double(v_)) return error_none; break;
-	    case TI_STRING: if (is_string(v_)) return error_none; break;
+	    case TI_BOOL:     if (is_bool(v_))     return error_none; break;
+	    case TI_INT:      if (is_int(v_)||(is_uint(v_) && boost::get<unsigned int>(v_) <= INT_MAX))
+                                                   return error_none; break;
+	    case TI_UINT:     if (is_uint(v_))     return error_none; break;
+	    case TI_DOUBLE:   if (is_double(v_))   return error_none; break;
+	    case TI_STRING:   if (is_string(v_))   return error_none; break;
+            case TI_TIME:     if (is_time(v_))     return error_none; break; 
+            case TI_IPV4ADDR: if (is_ipv4addr(v_)) return error_none; break;
+            case TI_IPV6ADDR: if (is_ipv6addr(v_)) return error_none; break;
 	    default: assert(false);
 	    }
 	    return type_mismatch;
