@@ -4,7 +4,7 @@ CONF4CPP=../../src/conf4cpp
 
 pass=0
 total=0
-for fname in *
+for fname in *.def
 do
 	$CONF4CPP $fname
 	if test $? -ne 255 
